package logging

import (
	"context"
	"time"

	"git.espin.casa/albert/cml04-gdm-int/pkg/api"
	"git.espin.casa/albert/cml04-l2-rsm-in/internal/service"
	"git.espin.casa/albert/logger"
)

type Logging struct {
	log  logger.LoggerAdapter
	next service.IService
}

// PublishBdRollData implements service.IService.
func (svc *Logging) PublishBdRollData(ctx context.Context, telegramData *api.BdRollData) (err error) {
	defer func(start time.Time) {
		logFields := logger.LogFields{
			"took":       time.Since(start),
			"bd_roll_id": telegramData.GetBdRollId(),
		}
		if err != nil {
			svc.log.Error("BD publish roll data failed", err, logFields)
		} else {
			svc.log.Info("BD publish roll data success", logFields)
		}
	}(time.Now())
	return svc.next.PublishBdRollData(ctx, telegramData)
}

// PublishEdRollData implements service.IService.
func (svc *Logging) PublishEdRollData(ctx context.Context, telegramData *api.EdRollData) (err error) {
	defer func(start time.Time) {
		logFields := logger.LogFields{
			"took":       time.Since(start),
			"ed_roll_id": telegramData.GetEdRollId(),
		}
		if err != nil {
			svc.log.Error("ED publish roll data failed", err, logFields)
		} else {
			svc.log.Info("ED publish roll data success", logFields)
		}
	}(time.Now())
	return svc.next.PublishEdRollData(ctx, telegramData)
}

// PublishUfRollData implements service.IService.
func (svc *Logging) PublishUfRollData(ctx context.Context, telegramData *api.UfRollData) (err error) {
	defer func(start time.Time) {
		logFields := logger.LogFields{
			"took":       time.Since(start),
			"uf_roll_id": telegramData.GetUfRollId(),
		}
		if err != nil {
			svc.log.Error("UF publish roll data failed", err, logFields)
		} else {
			svc.log.Info("UF publish roll data success", logFields)
		}
	}(time.Now())
	return svc.next.PublishUfRollData(ctx, telegramData)
}

// PublishUrRollData implements service.IService.
func (svc *Logging) PublishUrRollData(ctx context.Context, telegramData *api.UrRollData) (err error) {
	defer func(start time.Time) {
		logFields := logger.LogFields{
			"took":       time.Since(start),
			"ur_roll_id": telegramData.GetUrRollId(),
		}
		if err != nil {
			svc.log.Error("UR publish roll data failed", err, logFields)
		} else {
			svc.log.Info("UR publish roll data success", logFields)
		}
	}(time.Now())
	return svc.next.PublishUrRollData(ctx, telegramData)
}

// BdReqTelegram implements service.IService.
func (svc *Logging) BdReqTelegram(ctx context.Context, req *api.BdRollDataReq) (res *api.BdRollDataRes, err error) {
	defer func(start time.Time) {
		logFields := logger.LogFields{
			"took":            time.Since(start),
			"request_roll_id": req.GetRollId(),
		}
		if err != nil {
			svc.log.Error("BD request roll data failed", err, logFields)
		} else {
			svc.log.Info("BD request roll data success", logFields)
		}
	}(time.Now())
	return svc.next.BdReqTelegram(ctx, req)
}

// EdReqTelegram implements service.IService.
func (svc *Logging) EdReqTelegram(ctx context.Context, req *api.EdRollDataReq) (res *api.EdRollDataRes, err error) {
	defer func(start time.Time) {
		logFields := logger.LogFields{
			"took":            time.Since(start),
			"request_roll_id": req.GetRollId(),
		}
		if err != nil {
			svc.log.Error("ED request roll data failed", err, logFields)
		} else {
			svc.log.Info("ED request roll data success", logFields)
		}
	}(time.Now())
	return svc.next.EdReqTelegram(ctx, req)
}

// UfReqTelegram implements service.IService.
func (svc *Logging) UfReqTelegram(ctx context.Context, req *api.UfRollDataReq) (res *api.UfRollDataRes, err error) {
	defer func(start time.Time) {
		logFields := logger.LogFields{
			"took":            time.Since(start),
			"request_roll_id": req.GetRollId(),
		}
		if err != nil {
			svc.log.Error("UF request roll data failed", err, logFields)
		} else {
			svc.log.Info("UF request roll data success", logFields)
		}
	}(time.Now())
	return svc.next.UfReqTelegram(ctx, req)
}

// UrReqTelegram implements service.IService.
func (svc *Logging) UrReqTelegram(ctx context.Context, req *api.UrRollDataReq) (res *api.UrRollDataRes, err error) {
	defer func(start time.Time) {
		logFields := logger.LogFields{
			"took":            time.Since(start),
			"request_roll_id": req.GetRollId(),
		}
		if err != nil {
			svc.log.Error("UR request roll data failed", err, logFields)
		} else {
			svc.log.Info("UR request roll data success", logFields)
		}
	}(time.Now())
	return svc.next.UrReqTelegram(ctx, req)
}

func NewLoggingService(log logger.LoggerAdapter, next service.IService) service.IService {
	return &Logging{log, next}
}
